//NumeroPageEnTexte_F_504f.js
//© 07.04.2018 / Hans Haesler, Châtelard 52, CH-1018 Lausanne / hsa@ringier.ch
//DESCRIPTION:Remplace les numéros des pages par des lettres

// activer l'affichage des dialogues
app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;

// tester si un document est ouvert ////////////////////////////////////////////////
if (app.documents.length == 0) {
	alert ("Il n'y a pas de document ouvert.", "Attention");
	exit();
}
else {
	curDoc = app.documents[0];
}

// tester si des préfixes éventuels des sections sont contenus dans les numéros des pages
var allSections = curDoc.sections;
var nSections   = allSections.length;
for (var s=0; s<nSections; s++) {
	if (allSections[s].includeSectionPrefix) {
		alert ("Le préfixe d'une section ne doit pas être visible.", "Stop");
		exit();
	}
}

// les listes des chaînes de texte partiels ////////////////////////////////////////
oneStrings = ["zéro", "un", "deux", "trois", "quatre", "cinq", "six", "sept", "huit", "neuf"];
dblStrings = ["", "", "", "", "", "", "", "", "", "", "dix", "onze", "douze", "treize", "quatorze", "quinze", "seize", "dix-sept", "dix-huit", "dix-neuf"];
tenStrings = ["", "dix", "vingt", "trente", "quarante", "cinquante", "soixante"];

// effacer les préférences rechercher/remplacer ////////////////////////////////////
app.findGrepPreferences = app.changeGrepPreferences = null;
app.findTextPreferences = app.changeTextPreferences = null;

// préparer les chaînes de recherche ///////////////////////////////////////////////
app.findGrepPreferences.findWhat = "[0-9]+";
app.findTextPreferences.findWhat = "\u0018"; // = SpecialCharacters.AUTO_PAGE_NUMBER

// initialiser la liste des blocs de texte
var txtFrameArr = [];

// mémoriser et tester la sélection
var nTxtFrames = 0;
curSel = app.selection;
if (curSel.length > 0) {
	for (var s=0; s<curSel.length; s++) {
		var curType = curSel[s].constructor.name;
		if (curType == "TextFrame") {
			txtFrameArr.push(curSel[s]);
		}
		else if (curType == "Group") {
			var allItems = curSel[s].allPageItems;
			for (var n=0; n<allItems.length; n++) {
				var curType = allItems[n].constructor.name;
				if (curType == "TextFrame") {
					txtFrameArr.push(allItems[n]);
				}
			}
		}
		else {
			if (curSel[0].hasOwnProperty("baseline")) {
				txtFrameArr.push(curSel[0].parentTextFrames[0]);
			}
		}
	}
	nTxtFrames = txtFrameArr.length;
	if (nTxtFrames == 0) {
		alert ("La sélection ne contient pas de bloc de texte.", "Stop");
		app.findGrepPreferences = app.changeGrepPreferences = null;
		app.findTextPreferences = app.changeTextPreferences = null;
		exit();
	}
}

// vérifier le résultat ////////////////////////////////////////////////////////////
if (nTxtFrames == 0) { // s'il n'y a pas de sélection: appeler la fonction 
	// initialiser trois variables ///////////////////////////////////////////////////
	var allPages = curDoc.pages;
	var nPages   = allPages.length;
	var nSpreads = curDoc.masterSpreads.length;
	var firstMaster = curDoc.pages[0].appliedMaster;
	// vérifier le nombre des gabarits appliqués
	if (nSpreads > 1) {
		var moreThanOne = false;
		for (var n=1; n<nPages; n++) {
			if (curDoc.pages[n].appliedMaster != firstMaster) {
				moreThanOne = true;
				break;
			}
		}
		if (!moreThanOne) {
			nSpreads = 1;
		}
	}
	// appeler la fonction 
	getNotSel(allPages, nPages, nSpreads, firstMaster);
}
else { // traîter les blocs de la sélection
	for (tF=0; tF<nTxtFrames; tF++) {
		var curFrame = txtFrameArr[tF];
		var curCont  = curFrame.texts[0].contents;
		if (curCont.indexOf("\u0018") != -1) {
			pageName = curFrame.parentPage.name;
			var numberText = getNumberText(pageName);
			// lorsque la chaîne de texte n'est pas non définie...
			if (numberText != undefined) {
				app.changeTextPreferences.changeTo = numberText;
				curFrame.changeText();
			}
		}
		else {
			try {
				var oneNum = curFrame.texts[0].findGrep();
				var numCnt = oneNum[0].contents;
				var numberText = getNumberText(numCnt);
				if (numberText != undefined) {
					app.changeGrepPreferences.changeTo = numberText;
					oneNum[0].changeGrep();
				}
			}
			catch (e) {
			}
		}
	}
}

// effacer les préférences rechercher/remplacer ////////////////////////////////////
app.findGrepPreferences = app.changeGrepPreferences = null;
app.findTextPreferences = app.changeTextPreferences = null;

// informer l'utilisateur //////////////////////////////////////////////////////////
var dlog = new Window("palette");
dlog.size = [320,100];
dlog.add("statictext", [80, 100, 320, 116], "Terminé.");
dlog.show();
// une petite pause
$.sleep(999);
// fermer la fenêtre
dlog.close();


////////////////////////////////////////////////////////////////////////////////////
// trouver les occurrences //////////////////////////////////////////////////////////
function getNotSel(allPages, nPages, nSpreads, firstMaster) {
	var allNumbers = curDoc.findText();
	var nNumbers   = allNumbers.length;
	// quand il n'y a pas de blocs libérés (contenant un numéro de page)...
	if (nNumbers == 0) {
		// initialiser deux variables
		var pageNumberFrameL = "";
		var pageNumberFrameR = "";
		// les blocs texte des pages du gabarit (quand il n'y a qu'un gabarit)
		if (nSpreads == 1) {
			var sPages = firstMaster.pages.length;
			var allGroups = firstMaster.groups;
			var nGroups   = allGroups.length;
			// dissocier des groupes éventuels
			for (g=nGroups-1; g>-1; g--) {
				try {
					allGroups[g].ungroup();
				}
				catch (e) {
				}
			}
			var txtFramesL = firstMaster.pages[0].textFrames;
			var nFrames    = txtFramesL.length;
			for (var n=0; n<nFrames; n++) {
				var numFrame = txtFramesL[n];
				var curCont = numFrame.texts[0].contents;
				if (curCont.indexOf("\u0018") != -1) {
					var pageNumberFrameL = txtFramesL[n];
					break;
				}
			}
			if (sPages == 2) {
				txtFramesR = firstMaster.pages[1].textFrames;
				nFrames    = txtFramesR.length;
				for (var n=0; n<nFrames; n++) {
					var numFrame = txtFramesR[n];
					var curCont = numFrame.texts[0].contents;
					if (curCont.indexOf("\u0018") != -1) {
						var pageNumberFrameR = txtFramesR[n];
						break;
					}
				}
			}
		}
		// une boucle à travers les pages, libérer les blocs contenant un numéro de page automatique 
		// et remplacer les numéros des pages par le texte correspondant
		for (var p=0; p<nPages; p++) {
			// der Name der Seite
			var pNbr = (allPages[p].name);
			// appeler la fonction qui convertit les chiffres en texte
			var numberText = getNumberText(pNbr);
			// lorsque la chaîne de texte n'est non définie...
			if (numberText != undefined) {
				// mettre le bloc de la page de gauche ou de droite dans la variable 
				if (pNbr % 2 == 0 || sPages == 1) {
					var pageNumberFrame = pageNumberFrameL;
				}
				else {
					var pageNumberFrame = pageNumberFrameR;
				}
				try { // tenter de libérer le bloc texte et d'y insérer la chaîne de texte
					var curFrame = pageNumberFrame.override(allPages[p]);
					app.changeTextPreferences.changeTo = numberText;
					curFrame.changeText();
				}
				catch (e) {
				}
			}
		}
	}
	else { // sinon que les numéros des pages accessibles sur les pages du document
		for (var n=0; n<nNumbers; n++) {
			var curInfo  = getInfo(allNumbers[n]);
			var curFrame = curInfo[0];
			var curNbr   = curInfo[1];
			// appeler la fonction qui convertit les chiffres en texte
			var numberText = getNumberText(curNbr);
			// lorsque la chaîne de texte n'est non définie...
			if (numberText != undefined) {
				app.changeTextPreferences.changeTo = numberText;
				curFrame.changeText();
			}
		}
	}
	
	// quand il n'y a pas de blocs libérés et quand il y a plusieurs gabarits
	if (nNumbers == 0 && nSpreads > 1) {
		// une boucle à travers les pages, libérer les blocs contenant un numéro de page automatique 
		// et remplacer les numéros des pages par le texte correspondant
		for (var p=0; p<nPages; p++) {
			// trouver le gabarit appliqué à la page actuelle
			var actMaster = allPages[p].appliedMaster;
			// le nom de la page
			var pNbr = (allPages[p].name);
			// appeler la fonction qui convertit les chiffres en texte
			var numberText = getNumberText(pNbr);
			// lorsque la chaîne de texte n'est pas non définie...
			if (numberText != undefined) {
				var allGroups = actMaster.groups;
				var nGroups   = allGroups.length;
				// dissocier des groupes éventuels
				for (g=nGroups-1; g>-1; g--) {
					try {
						allGroups[g].ungroup();
					}
					catch (e) {
					}
				}
				var sPages = actMaster.pages.length;
				// mettre le bloc de la page de gauche ou de droite dans la variable 
				if (pNbr % 2 == 0 || sPages == 1) {
					var txtFramesL = actMaster.pages[0].textFrames;
					var nFrames    = txtFramesL.length;
					for (var n=0; n<nFrames; n++) {
						var numFrame = txtFramesL[n];
						try {
							var curCont = numFrame.texts[0].contents;
							if (curCont.indexOf("\u0018") != -1) {
								var pageNumberFrame = txtFramesL[n];
								break;
							}
						}
						catch (e) {
						}
					}
				}
				else {
					var txtFramesR = actMaster.pages[1].textFrames;
					var nFrames    = txtFramesR.length;
					for (var n=0; n<nFrames; n++) {
						var numFrame = txtFramesR[n];
						var curCont = numFrame.texts[0].contents;
						if (curCont.indexOf("\u0018") != -1) {
							var pageNumberFrame = txtFramesR[n];
							break;
						}
					}
				}
				try { // tenter de libérer le bloc texte et d'y insérer la chaîne de texte
					var curFrame = pageNumberFrame.override(allPages[p]);
					app.changeTextPreferences.changeTo = numberText;
					curFrame.changeText();
				}
				catch (e) {
				}
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////
// aller chercher le numéro de la page /////////////////////////////////////////////
function getInfo(curNum) {
	var pageName = -1;
	try {
		curFrame = curNum.parent.textContainers[0];
		pageName = curFrame.parentPage.name;
	}
	catch (e) {
	}
	return [curFrame, pageName];	
}

////////////////////////////////////////////////////////////////////////////////////
// la fonction qui convertit les chiffres en texte /////////////////////////////////
function getNumberText(pNbr) {
	if (isNaN(pNbr)) {
		alert ("Ce script ne fonctionne qu'avec des chiffres arabes.", "Attention");
		app.findGrepPreferences = app.changeGrepPreferences = null;
		app.findTextPreferences = app.changeTextPreferences = null;
		exit();
	}
	var strLen = pNbr.length;
	var strVal = pNbr*1;
	if (strLen == 1) {
		if (strVal == 1) {
			return "une";
		}
		else {
			return (oneStrings[strVal]);
		}
	}
	else if (strLen == 2) {
		if (strVal < 20) {
			return (dblStrings[strVal]);
		}
		else if (strVal > 19) {
			if (pNbr[0]*1 < 7) {
				if (pNbr[1] == "0") {
					return tenStrings[pNbr[0]*1];
				}
				else if (pNbr[1] == "1") {
					return (tenStrings[pNbr[0]*1])+" et un";
				}
				else {
					return (tenStrings[pNbr[0]*1])+"-"+(oneStrings[pNbr[1]*1]);
				}
			}
			else {
				if (pNbr[0] == "7") {
					if (pNbr[1] == "0") {
						return "soixante-dix";
					}
					else {
						return "soixante-"+(dblStrings[pNbr[1]*1+10]);
					}
				}
				else if (pNbr[0] == "8") {
					if (pNbr[1] == "0") {
						return "quatre-vingts";
					}
					else {
						return "quatre-vingt-"+(oneStrings[pNbr[1]*1]);
					}
				}
				else if (pNbr[0] == "9") {
					if (pNbr[1] == "0") {
						return "quatre-vingt-dix";
					}
					else {
						return "quatre-vingt-"+(dblStrings[pNbr[1]*1+10]);
					}
				}
			}
		}
	}
	else if (strLen == 3) {
		var rstVal = strVal % 100;
		if (rstVal == 0) {
			if (pNbr[0] == "1") {
				return "cent";
			}
			else {
				return (oneStrings[pNbr[0]*1])+" cents";
			}
		}
		else if (rstVal == 1) {
			if (pNbr[0] == "1") {
				return "cent un";
			}
			else {
				return (oneStrings[pNbr[0]*1])+" cent un";
			}
		}
		else if (rstVal < 10) {
			if (pNbr[0] == "1") {
				return "cent "+(oneStrings[(pNbr[2]*1)]);
			}
			else {
				return (oneStrings[pNbr[0]*1])+" cent "+(oneStrings[(pNbr[2]*1)]);
			}
		}
		else if (rstVal < 20) {
			if (pNbr[0] == "1") {
				return "cent "+(dblStrings[(pNbr[1]+pNbr[2])*1]);
			}
			else {
				return (oneStrings[pNbr[0]*1])+" cent "+(dblStrings[(pNbr[1]+pNbr[2])*1]);
			}
		}
		else if (rstVal > 19) {
			if (pNbr[1]*1 < 7) {
				if (pNbr[0] == "1") {
					if (pNbr[2] == "0") {
						return "cent "+tenStrings[pNbr[1]*1];
					}
					else if (pNbr[2] == "1") {
						return "cent "+(tenStrings[pNbr[1]*1])+" et un";
					}
					else {
						return "cent "+(tenStrings[pNbr[1]*1])+"-"+(oneStrings[pNbr[2]*1]);
					}
				}
				else {
					if (pNbr[2] == "0") {
						return (oneStrings[pNbr[0]*1])+" cent "+tenStrings[pNbr[1]*1];
					}
					else if (pNbr[2] == "1") {
						return (oneStrings[pNbr[0]*1])+" cent "+(tenStrings[pNbr[1]*1])+" et un";
					}
					else {
						return (oneStrings[pNbr[0]*1])+" cent "+(tenStrings[pNbr[1]*1])+"-"+(oneStrings[pNbr[2]*1]);
					}
				}
			}
			else {
				if (pNbr[1] == "7") {
					if (pNbr[2] == "0") {
						if (pNbr[0] == "1") {
							return "cent "+"soixante-dix";
						}
						else {
							return (oneStrings[pNbr[0]*1])+" cent "+"soixante-dix";
						}
					}
					else {
						if (pNbr[0] == "1") {
							return "cent "+"soixante-"+(dblStrings[pNbr[2]*1+10]);
						}
						else {
							return (oneStrings[pNbr[0]*1])+" cent "+"soixante-"+(dblStrings[pNbr[2]*1+10]);
						}
					}
				}
				else if (pNbr[1] == "8") {
					if (pNbr[2] == "0") {
						if (pNbr[0] == "1") {
							return "cent "+"quatre-vingts";
						}
						else {
							return (oneStrings[pNbr[0]*1])+" cent "+"quatre-vingts";
						}
					}
					else {
						if (pNbr[0] == "1") {
							return "cent "+"quatre-vingt-"+(oneStrings[pNbr[2]*1]);
						}
						else {
							return (oneStrings[pNbr[0]*1])+" cent "+"quatre-vingt-"+(oneStrings[pNbr[2]*1]);
						}
					}
				}
				else if (pNbr[1] == "9") {
					if (pNbr[2] == "0") {
						if (pNbr[0] == "1") {
							return "cent "+"quatre-vingt-dix";
						}
						else {
							return (oneStrings[pNbr[0]*1])+" cent "+"quatre-vingt-dix";
						}
					}
					else {
						if (pNbr[0] == "1") {
							return "cent "+"quatre-vingt-"+(dblStrings[pNbr[2]*1+10]);
						}
						else {
							return (oneStrings[pNbr[0]*1])+" cent "+"quatre-vingt-"+(dblStrings[pNbr[2]*1+10]);
						}
					}
				}
			}
		}
	}
	else if (strLen == 4) {
		if (pNbr == 1000) {
			return "mille";
		}
		else {
			alert ("Ce script ne peut pas convertir des valeurs à partir de 1001.", "Attention");
			app.findGrepPreferences = app.changeGrepPreferences = null;
			app.findTextPreferences = app.changeTextPreferences = null;
			exit();
		}
	}
}